/*#########################################################################
#   Amarok script for interfacing with SoundCloud                         #
#                                                                         #
#   Copyright                                                             #
#   (C) 2014 Be <be.0@gmx.com>                                            #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 3 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   Note that if you modify this software and continue to use the         #
#   SoundCloud API in your modified version, the SoundCloud API Terms of  #
#   Use (https://developers.soundcloud.com/docs/api/terms-of-use ) state, #
#   in the section General Prohibitions, clause xiv:                      #
#      You must not use or attempt to use another person's client ID      #
#      and/or client secret, unless you are working on the same app.      #
#   You are encouraged to communicate your modifications to the author    #
#   so they can be incorporated into a future version of this program.    #
#   If you wish to publish a modified version, to comply with the         #
#   SoundCloud API Terms of Use, register your new program with           #
#   SoundCloud through their website and replace the value of the global  #
#   variable client_id, defined at the first line of code, with your      #
#   client ID.                                                            #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         #
#########################################################################*/

var client_id = '182a6730c85291f153ce073e55f0ad9e'

Importer.loadQtBinding( "qt.core" );

function SoundCloudService() {
	ScriptableServiceScript.call( this, "SoundCloud Integration", 3, "Stream from SoundCloud", "Stream from SoundCloud", true );
}

function onConfigure() {
	// TODO: Username and password dialog
	Amarok.alert( "This script does not require any configuration." );
}

function htmlify (string) {
	string = string.replace(/(?:\r\n|\r|\n)/g,'<br/>');
	// URL regex from Android's WEB_URL constant
	urlRegex = /((?:(http|https|Http|Https|rtsp|Rtsp):\/\/(?:(?:[a-zA-Z0-9\$\-\_\.\+\!\*\'\(\)\,\;\?\&\=]|(?:\%[a-fA-F0-9]{2})){1,64}(?:\:(?:[a-zA-Z0-9\$\-\_\.\+\!\*\'\(\)\,\;\?\&\=]|(?:\%[a-fA-F0-9]{2})){1,25})?\@)?)?((?:(?:[a-zA-Z0-9][a-zA-Z0-9\-]{0,64}\.)+(?:(?:aero|arpa|asia|a[cdefgilmnoqrstuwxz])|(?:biz|b[abdefghijmnorstvwyz])|(?:cat|com|coop|c[acdfghiklmnoruvxyz])|d[ejkmoz]|(?:edu|e[cegrstu])|f[ijkmor]|(?:gov|g[abdefghilmnpqrstuwy])|h[kmnrtu]|(?:info|int|i[delmnoqrst])|(?:jobs|j[emop])|k[eghimnrwyz]|l[abcikrstuvy]|(?:mil|mobi|museum|m[acdghklmnopqrstuvwxyz])|(?:name|net|n[acefgilopruz])|(?:org|om)|(?:pro|p[aefghklmnrstwy])|qa|r[eouw]|s[abcdeghijklmnortuvyz]|(?:tel|travel|t[cdfghjklmnoprtvwz])|u[agkmsyz]|v[aceginu]|w[fs]|y[etu]|z[amw]))|(?:(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9])\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9]|0)\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9]|0)\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[0-9])))(?:\:\d{1,5})?)(\/(?:(?:[a-zA-Z0-9\;\/\?\:\@\&\=\#\~\-\.\+\!\*\'\(\)\,\_])|(?:\%[a-fA-F0-9]{2}))*)?(?:\b|$)/gi;
	string = string.replace(urlRegex, function(url) {
		if ( ( url.indexOf(".jpg") > 0 ) || ( url.indexOf(".png") > 0 ) || ( url.indexOf(".gif") > 0 ) ) {
			return '<img src="' + url + '">';
		} else {
			return '<a href="' + url + '">' + url + '</a>';
		}
	}) 
	return string;
}

var playlistTracks = []

function playlists(reply) {
	plist = JSON.parse(reply);
	playlistTracks = [];
	if (! ('errors' in plist)){
		item = Amarok.StreamItem;
		item.level = 1;
		len = plist.length;
		for ( i = 0; i < len; i++ ) {
			item.callbackData = JSON.stringify(plist[i].tracks);
			item.itemName = plist[i].title;
			item.artist = plist[i].user.username;
			item.year = plist[i].release_year;
			item.infoHtml = htmlify(plist[i].description);
			item.coverUrl = plist[i].artwork_url;
			script.insertItem(item);
			for (j in plist[i].tracks) {
				playlistTracks.push(plist[i].tracks[j]);
			}
		}
	}
	new Downloader(new QUrl('https://api.soundcloud.com/users/' + plist[0].user.username + '/tracks?client_id=' + client_id ), nonPlaylistTracks);
}

function nonPlaylistTracks (reply) {
	tracklist = JSON.parse(reply);
	item = Amarok.StreamItem;
	item.level = 1;
	item.callbackData = JSON.stringify(tracklist.filter(notInPlaylist));
	item.itemName = 'Other tracks';
	item.artist = tracklist[0].user.username;
	item.coverUrl = tracklist[0].user.avatar_url;
	script.insertItem(item);
	script.donePopulating();
}

function notInPlaylist (value, index, array) {
	for (i in playlistTracks) {
		if (value.title == playlistTracks[i].title) {
			return false;
		}
	}
	return true;
}

function tracks(callback) {
	tracklist = JSON.parse(callback);
	if (! ('errors' in tracklist)){
		item = Amarok.StreamItem;
		item.level = 0;
		for (i in tracklist) {
			item.callbackData = '';
			item.itemName = tracklist[i].title;
			item.artist = tracklist[i].user.username;
			item.year = tracklist[i].release_year;
			item.playableUrl = tracklist[i].stream_url + '?client_id=' + client_id;
			item.infoHtml = trackInfo(tracklist[i]);
			item.coverUrl = tracklist[i].artwork_url;
			script.insertItem(item);
		}
		script.donePopulating();
	} else {
		item = Amarok.StreamItem;
		item.level = 0;
		item.itemName = 'Search';
		item.callbackData = '';
		script.insertItem(item);
		script.donePopulating();
	}
	
}

function trackInfo (info) {
	var br = '<br/>'
	return '' +
	'<div style="text-align:center;">' +
		'<a href="' + info.user.permalink_url + '">' +
			'<img src="' + info.user.avatar_url + '" style="position:absolute;top:0;right:0;"/>' +
			'<h1>' + info.user.username + '</h1>' +
		'</a>' +
		'<h2><a href="' + info.permalink_url + '">' + info.title + '</a></h2>' +
		'Published: ' + info.created_at + br +
		'Plays: ' + info.playback_count + ' | ' +
		'Favorites: ' + info.favoritings_count + ' | ' +
		'Tags: ' + info.tag_list +
		'</div>' + br +
	'<div>' + htmlify(info.description) + '</div>';
	//TODO: comments
}

function artist(reply) {
	data = JSON.parse(reply);
	
	if ((! ('errors' in data)) && data.track_count > 0) {
		item = Amarok.StreamItem;
		item.level = 2;
		item.callbackData = reply;
		item.itemName = data.username;
		item.infoHtml = htmlify(data.description);
		script.insertItem(item);
		script.donePopulating();
		playlistTracks[data.username] = []
	} else {
		prompt();
	}
}

function prompt() {
	item = Amarok.StreamItem;
	item.level = 2;
	item.itemName = 'Search for an artist...';
	item.callbackData = '{"track_count": 0}';
	item.coverUrl = '';
	item.playableUrl = '';
	item.infoHtml = '';
	script.insertItem(item);
	script.donePopulating();
}

function onPopulating( level, callback, filter ) {
	Amarok.debug("Populating level " + level);
	switch (level) {
		case 2:
			if (filter == '') {
				prompt();
			} else {
				new Downloader(new QUrl('https://api.soundcloud.com/users/' + filter + '?client_id=' + client_id ), artist);
			}
			break;
		case 1:
			artistData = JSON.parse(callback);
			if ((! ('errors' in artistData)) && artistData.track_count > 0) {
				new Downloader(new QUrl('https://api.soundcloud.com/users/' + artistData.username + '/playlists?client_id=' + client_id ), playlists);
			} else {
				item = Amarok.StreamItem;
				item.level = 1;
				item.itemName = 'Search for an artist...';
				item.callbackData = '{"errors": "blah"}';
				item.playableUrl = '';
				item.coverUrl = '';
				item.infoHtml = '';
				script.insertItem(item);
				script.donePopulating();
			}
			break;
		case 0:
			tracks(callback);
			break;
	}
}

Amarok.configured.connect( onConfigure );
script = new SoundCloudService();
script.populate.connect( onPopulating );
